@@ -66,28 +66,30 @@ type Config struct {
 }
 
 type serverConfig struct {
-	AdminIDs           []string           `hcl:"admin_ids"`
-	AgentTTL           string             `hcl:"agent_ttl"`
-	AuditLogEnabled    bool               `hcl:"audit_log_enabled"`
-	BindAddress        string             `hcl:"bind_address"`
-	BindPort           int                `hcl:"bind_port"`
-	CAKeyType          string             `hcl:"ca_key_type"`
-	CASubject          *caSubjectConfig   `hcl:"ca_subject"`
-	CATTL              string             `hcl:"ca_ttl"`
-	DataDir            string             `hcl:"data_dir"`
-	DefaultX509SVIDTTL string             `hcl:"default_x509_svid_ttl"`
-	DefaultJWTSVIDTTL  string             `hcl:"default_jwt_svid_ttl"`
-	Experimental       experimentalConfig `hcl:"experimental"`
-	Federation         *federationConfig  `hcl:"federation"`
-	JWTIssuer          string             `hcl:"jwt_issuer"`
-	JWTKeyType         string             `hcl:"jwt_key_type"`
-	LogFile            string             `hcl:"log_file"`
-	LogLevel           string             `hcl:"log_level"`
-	LogFormat          string             `hcl:"log_format"`
-	LogSourceLocation  bool               `hcl:"log_source_location"`
-	RateLimit          rateLimitConfig    `hcl:"ratelimit"`
-	SocketPath         string             `hcl:"socket_path"`
-	TrustDomain        string             `hcl:"trust_domain"`
+	AdminIDs                     []string           `hcl:"admin_ids"`
+	AgentTTL                     string             `hcl:"agent_ttl"`
+	AuditLogEnabled              bool               `hcl:"audit_log_enabled"`
+	BindAddress                  string             `hcl:"bind_address"`
+	BindPort                     int                `hcl:"bind_port"`
+	CAKeyType                    string             `hcl:"ca_key_type"`
+	CASubject                    *caSubjectConfig   `hcl:"ca_subject"`
+	CATTL                        string             `hcl:"ca_ttl"`
+	DataDir                      string             `hcl:"data_dir"`
+	DefaultX509SVIDTTL           string             `hcl:"default_x509_svid_ttl"`
+	DefaultJWTSVIDTTL            string             `hcl:"default_jwt_svid_ttl"`
+	Experimental                 experimentalConfig `hcl:"experimental"`
+	Federation                   *federationConfig  `hcl:"federation"`
+	JWTIssuer                    string             `hcl:"jwt_issuer"`
+	JWTKeyType                   string             `hcl:"jwt_key_type"`
+	LogFile                      string             `hcl:"log_file"`
+	LogLevel                     string             `hcl:"log_level"`
+	LogFormat                    string             `hcl:"log_format"`
+	LogSourceLocation            bool               `hcl:"log_source_location"`
+	PruneAttestedNodesExpiredFor string             `hcl:"prune_attested_nodes_expired_for"`
+	PruneNonReattestableNodes    bool               `hcl:"prune_tofu_nodes"`
+	RateLimit                    rateLimitConfig    `hcl:"ratelimit"`
+	SocketPath                   string             `hcl:"socket_path"`
+	TrustDomain                  string             `hcl:"trust_domain"`
 
 	ConfigPath string
 	ExpandEnv  bool
@@ -98,21 +100,18 @@ type serverConfig struct {
 	ProfilingFreq    int      `hcl:"profiling_freq"`
 	ProfilingNames   []string `hcl:"profiling_names"`
 
-	// Temporary configurables
-	// UseLegacyDownstreamX509CATTL is deprecated and should be removed in SPIRE 1.12.0.
-	UseLegacyDownstreamX509CATTL *bool `hcl:"use_legacy_downstream_x509_ca_ttl"`
-
 	UnusedKeyPositions map[string][]token.Pos `hcl:",unusedKeyPositions"`
 }
 
 type experimentalConfig struct {
-	AuthOpaPolicyEngine   *authpolicy.OpaEngineConfig `hcl:"auth_opa_policy_engine"`
-	CacheReloadInterval   string                      `hcl:"cache_reload_interval"`
-	EventsBasedCache      bool                        `hcl:"events_based_cache"`
-	PruneEventsOlderThan  string                      `hcl:"prune_events_older_than"`
-	EventTimeout          string                      `hcl:"event_timeout"`
-	SQLTransactionTimeout string                      `hcl:"sql_transaction_timeout"`
-	RequirePQKEM          bool                        `hcl:"require_pq_kem"`
+	AuthOpaPolicyEngine     *authpolicy.OpaEngineConfig `hcl:"auth_opa_policy_engine"`
+	CacheReloadInterval     string                      `hcl:"cache_reload_interval"`
+	FullCacheReloadInterval string                      `hcl:"full_cache_reload_interval"`
+	EventsBasedCache        bool                        `hcl:"events_based_cache"`
+	PruneEventsOlderThan    string                      `hcl:"prune_events_older_than"`
+	EventTimeout            string                      `hcl:"event_timeout"`
+	SQLTransactionTimeout   string                      `hcl:"sql_transaction_timeout"`
+	RequirePQKEM            bool                        `hcl:"require_pq_kem"`
 
 	Flags fflag.RawConfig `hcl:"feature_flags"`
 
@@ -568,26 +567,13 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		sc.CATTL = ttl
 	}
 
-	if c.Server.UseLegacyDownstreamX509CATTL != nil {
-		sc.Log.Warn("'use_legacy_downstream_x509_ca_ttl' is deprecated and will be removed in a future release")
-		sc.UseLegacyDownstreamX509CATTL = *c.Server.UseLegacyDownstreamX509CATTL
-		if sc.UseLegacyDownstreamX509CATTL {
-			sc.Log.Warn("Using legacy downstream X509 CA TTL calculation; this option will be removed in a future release")
-		} else {
-			sc.Log.Info("Using preferred downstream X509 CA TTL calculation")
-		}
-	} else {
-		// The flag should be removed in SPIRE 1.13.0.
-		sc.UseLegacyDownstreamX509CATTL = false
-		sc.Log.Info("Using preferred downstream X509 CA TTL calculation")
-	}
-
 	// If the configured TTLs can lead to surprises, then do our best to log an
 	// accurate message and guide the user to resolution
-	ttlChecks := []struct {
+	type ttlCheck struct {
 		name string
 		ttl  time.Duration
-	}{
+	}
+	ttlChecks := []ttlCheck{
 		{
 			name: "default_x509_svid_ttl",
 			ttl:  sc.X509SVIDTTL,
@@ -597,6 +583,12 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 			ttl:  sc.JWTSVIDTTL,
 		},
 	}
+	if sc.AgentTTL != 0 {
+		ttlChecks = append(ttlChecks, ttlCheck{
+			name: "agent_ttl",
+			ttl:  sc.AgentTTL,
+		})
+	}
 
 	for _, ttlCheck := range ttlChecks {
 		if !hasCompatibleTTL(sc.CATTL, ttlCheck.ttl) {
@@ -681,6 +673,17 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 	sc.Telemetry = c.Telemetry
 	sc.HealthChecks = c.HealthChecks
 
+	if c.Server.PruneAttestedNodesExpiredFor != "" {
+		expiredFor, err := time.ParseDuration(c.Server.PruneAttestedNodesExpiredFor)
+		if err != nil {
+			return nil, fmt.Errorf("could not parse prune_attested_nodes_expired_for: %w", err)
+		}
+		sc.PruneAttestedNodesExpiredFor = expiredFor
+		if c.Server.PruneNonReattestableNodes {
+			sc.PruneNonReattestableNodes = c.Server.PruneNonReattestableNodes
+		}
+	}
+
 	if !allowUnknownConfig {
 		if err := checkForUnknownConfig(c, sc.Log); err != nil {
 			return nil, err
@@ -699,6 +702,14 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		sc.CacheReloadInterval = interval
 	}
 
+	if c.Server.Experimental.FullCacheReloadInterval != "" {
+		interval, err := time.ParseDuration(c.Server.Experimental.FullCacheReloadInterval)
+		if err != nil {
+			return nil, fmt.Errorf("could not parse full cache reload interval: %w", err)
+		}
+		sc.FullCacheReloadInterval = interval
+	}
+
 	if c.Server.Experimental.PruneEventsOlderThan != "" {
 		interval, err := time.ParseDuration(c.Server.Experimental.PruneEventsOlderThan)
 		if err != nil {
@@ -772,10 +783,7 @@ func setBundleEndpointConfigProfile(config *bundleEndpointConfig, dataDir string
 			return nil
 		case profileConfig.HTTPSWeb.ServingCertFile != nil:
 			federationConfig.BundleEndpoint.DiskCertManager, err = configToDiskCertManager(profileConfig.HTTPSWeb.ServingCertFile, log)
-			if err != nil {
-				return err
-			}
-			return nil
+			return err
 		default:
 			return errors.New("malformed https_web profile configuration: 'acme' or 'serving_cert_file' is required")
 		}
