@@ -7,6 +7,46 @@ import (
 	"sync"
 )
 
+type TaskRunner struct {
+	wg     sync.WaitGroup
+	ctx    context.Context
+	cancel context.CancelCauseFunc
+}
+
+func NewTaskRunner(ctx context.Context, cancel context.CancelCauseFunc) *TaskRunner {
+	return &TaskRunner{
+		ctx:    ctx,
+		cancel: cancel,
+	}
+}
+
+func (t *TaskRunner) StartTasks(tasks ...func(context.Context) error) {
+	runTask := func(task func(context.Context) error) (err error) {
+		defer func() {
+			if r := recover(); r != nil {
+				err = fmt.Errorf("panic: %v\n%s", r, string(debug.Stack()))
+			}
+			t.wg.Done()
+		}()
+		return task(t.ctx)
+	}
+
+	t.wg.Add(len(tasks))
+	for _, task := range tasks {
+		go func() {
+			err := runTask(task)
+			if err != nil {
+				t.cancel(err)
+			}
+		}()
+	}
+}
+
+func (t *TaskRunner) Wait() error {
+	t.wg.Wait()
+	return context.Cause(t.ctx)
+}
+
 // RunTasks executes all the provided functions concurrently and waits for
 // them all to complete. If a function returns an error, all other functions
 // are canceled (i.e. the context they are passed is canceled) and the error is
@@ -16,43 +56,8 @@ import (
 // RunTasks MUST support cancellation via the provided context for RunTasks to
 // work properly.
 func RunTasks(ctx context.Context, tasks ...func(context.Context) error) error {
-	var wg sync.WaitGroup
-	ctx, cancel := context.WithCancel(ctx)
-	defer func() {
-		cancel()
-		wg.Wait()
-	}()
-
-	errch := make(chan error, len(tasks))
-
-	runTask := func(task func(context.Context) error) (err error) {
-		defer func() {
-			if r := recover(); r != nil {
-				err = fmt.Errorf("panic: %v\n%s\n", r, string(debug.Stack()))
-			}
-			wg.Done()
-		}()
-		return task(ctx)
-	}
-
-	wg.Add(len(tasks))
-	for _, task := range tasks {
-		go func() {
-			errch <- runTask(task)
-		}()
-	}
-
-	for complete := 0; complete < len(tasks); {
-		select {
-		case <-ctx.Done():
-			return ctx.Err()
-		case err := <-errch:
-			if err != nil {
-				return err
-			}
-			complete++
-		}
-	}
-
-	return nil
+	nctx, cancel := context.WithCancelCause(ctx)
+	t := NewTaskRunner(nctx, cancel)
+	t.StartTasks(tasks...)
+	return t.Wait()
 }
