@@ -34,7 +34,6 @@ type OpaEngineConfig struct {
 type LocalOpaProviderConfig struct {
 	RegoPath       string `hcl:"rego_path"`
 	PolicyDataPath string `hcl:"policy_data_path"`
-	UseRegoV1      bool   `hcl:"use_rego_v1"`
 }
 
 // Input represents context associated with an access request.
@@ -65,12 +64,12 @@ func NewEngineFromConfigOrDefault(ctx context.Context, logger logrus.FieldLogger
 	if cfg == nil {
 		return DefaultAuthPolicy(ctx)
 	}
-	return newEngine(ctx, logger, cfg)
+	return newEngine(ctx, cfg)
 }
 
 // newEngine returns a new policy engine. Or nil if no
 // config is provided.
-func newEngine(ctx context.Context, logger logrus.FieldLogger, cfg *OpaEngineConfig) (*Engine, error) {
+func newEngine(ctx context.Context, cfg *OpaEngineConfig) (*Engine, error) {
 	switch {
 	case cfg == nil:
 		return nil, errors.New("policy engine configuration is nil")
@@ -102,24 +101,17 @@ func newEngine(ctx context.Context, logger logrus.FieldLogger, cfg *OpaEngineCon
 		store = inmem.NewFromObject(map[string]any{})
 	}
 
-	version := ast.RegoV0
-	if cfg.LocalOpaProvider.UseRegoV1 {
-		version = ast.RegoV1
-	} else {
-		logger.Warn("Using rego.v0 policy format, which will be depracated in SPIRE 1.13; Update the policy to rego.v1 and specify 'use_rego_v1 = true' in the configuration.")
-	}
-
-	return NewEngineFromRego(ctx, string(module), store, version)
+	return NewEngineFromRego(ctx, string(module), store)
 }
 
 // NewEngineFromRego is a helper to create the Engine object
-func NewEngineFromRego(ctx context.Context, regoPolicy string, dataStore storage.Store, version ast.RegoVersion) (*Engine, error) {
+func NewEngineFromRego(ctx context.Context, regoPolicy string, dataStore storage.Store) (*Engine, error) {
 	rego := rego.New(
 		rego.Query("data.spire.result"),
 		rego.Package("spire"),
 		rego.Module("spire.rego", regoPolicy),
 		rego.Store(dataStore),
-		rego.SetRegoVersion(version),
+		rego.SetRegoVersion(ast.RegoV1),
 	)
 	pr, err := rego.PartialResult(ctx)
 	if err != nil {
