@@ -11,6 +11,8 @@ import (
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	"github.com/spiffe/spire/pkg/common/idutil"
 	"github.com/spiffe/spire/pkg/common/telemetry"
+	"github.com/spiffe/spire/pkg/server/api"
+	"github.com/spiffe/spire/pkg/server/authorizedentries"
 	"github.com/spiffe/spire/pkg/server/datastore"
 	"github.com/spiffe/spire/proto/spire/common"
 	"github.com/spiffe/spire/test/clock"
@@ -20,14 +22,23 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
-func TestNewAuthorizedEntryFetcherWithEventsBasedCache(t *testing.T) {
+func TestNewAuthorizedEntryFetcherEvents(t *testing.T) {
 	ctx := context.Background()
 	log, _ := test.NewNullLogger()
 	clk := clock.NewMock(t)
 	ds := fakedatastore.New(t)
 	metrics := fakemetrics.New()
 
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	assert.NoError(t, err)
 	assert.NotNil(t, ef)
 
@@ -81,7 +92,7 @@ func TestNewAuthorizedEntryFetcherWithEventsBasedCache(t *testing.T) {
 	assert.NoError(t, err)
 
 	// Create one registration entry parented to the agent directly
-	_, err = ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+	entry1, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
 		SpiffeId: "spiffe://example.org/viaagent",
 		ParentId: agentID.String(),
 		Selectors: []*common.Selector{
@@ -94,7 +105,7 @@ func TestNewAuthorizedEntryFetcherWithEventsBasedCache(t *testing.T) {
 	assert.NoError(t, err)
 
 	// Create one registration entry parented to the alias
-	_, err = ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+	entry2, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
 		SpiffeId: "spiffe://example.org/viaalias",
 		ParentId: "spiffe://example.org/alias",
 		Selectors: []*common.Selector{
@@ -111,7 +122,7 @@ func TestNewAuthorizedEntryFetcherWithEventsBasedCache(t *testing.T) {
 
 	entries, err := ef.FetchAuthorizedEntries(ctx, agentID)
 	assert.NoError(t, err)
-	assert.Equal(t, 2, len(entries))
+	compareEntries(t, entries, entry1, entry2)
 
 	// Assert metrics
 	expectedMetrics := []fakemetrics.MetricItem{
@@ -126,7 +137,7 @@ func TestNewAuthorizedEntryFetcherWithEventsBasedCache(t *testing.T) {
 	assert.ElementsMatch(t, expectedMetrics, metrics.AllMetrics(), "should emit metrics for node aliases, entries, and agents")
 }
 
-func TestNewAuthorizedEntryFetcherWithEventsBasedCacheErrorBuildingCache(t *testing.T) {
+func TestNewAuthorizedEntryFetcherEventsErrorBuildingCache(t *testing.T) {
 	ctx := context.Background()
 	log, _ := test.NewNullLogger()
 	clk := clock.NewMock(t)
@@ -136,7 +147,16 @@ func TestNewAuthorizedEntryFetcherWithEventsBasedCacheErrorBuildingCache(t *test
 	buildErr := errors.New("build error")
 	ds.SetNextError(buildErr)
 
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	assert.Error(t, err)
 	assert.Nil(t, ef)
 
@@ -178,9 +198,14 @@ func TestBuildCacheSavesSkippedEvents(t *testing.T) {
 	})
 	require.NoError(t, err)
 
-	_, registrationEntries, attestedNodes, err := buildCache(ctx, log, metrics, ds, clk, defaultCacheReloadInterval, defaultEventTimeout)
+	cache := authorizedentries.NewCache(clk)
+
+	registrationEntries, err := buildRegistrationEntriesCache(ctx, log, metrics, ds, clk, cache, pageSize, defaultCacheReloadInterval, defaultEventTimeout)
 	require.NoError(t, err)
 	require.NotNil(t, registrationEntries)
+
+	attestedNodes, err := buildAttestedNodesCache(ctx, log, metrics, ds, clk, cache, defaultCacheReloadInterval, defaultEventTimeout)
+	require.NoError(t, err)
 	require.NotNil(t, attestedNodes)
 
 	assert.Contains(t, registrationEntries.eventTracker.events, uint(2))
@@ -205,6 +230,61 @@ func TestBuildCacheSavesSkippedEvents(t *testing.T) {
 	assert.ElementsMatch(t, expectedMetrics, metrics.AllMetrics(), "should emit no metrics")
 }
 
+func TestRunUpdateCacheTaskDoesFullUpdate(t *testing.T) {
+	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+	log, _ := test.NewNullLogger()
+	log.SetLevel(logrus.DebugLevel)
+	clk := clock.NewMock(t)
+	ds := fakedatastore.New(t)
+	metrics := fakemetrics.New()
+
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     3 * time.Second,
+		fullCacheReloadInterval: 5 * time.Second,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
+	require.NoError(t, err)
+	require.NotNil(t, ef)
+
+	ef.mu.RLock()
+	initialCache := ef.cache
+	ef.mu.RUnlock()
+
+	// Start Update Task
+	updateCacheTaskErr := make(chan error)
+	go func() {
+		updateCacheTaskErr <- ef.RunUpdateCacheTask(ctx)
+	}()
+	clk.WaitForTickerMulti(time.Second, 2, "waiting to create tickers")
+
+	// First iteration, cache should not be rebuilt
+	clk.Add(4 * time.Second)
+	ef.mu.RLock()
+	require.Equal(t, initialCache, ef.cache)
+	ef.mu.RUnlock()
+
+	// Second iteration, cache should be rebuilt
+	// First we wait for the fullCacheReloadTicker to
+	// set the fullCacheReload flag to true
+	clk.Add(5 * time.Second)
+	// And then once a gain wait some more for the
+	// cache reload ticker to tick again.
+	clk.Add(6 * time.Second)
+	ef.mu.RLock()
+	require.NotEqual(t, initialCache, ef.cache)
+	ef.mu.RUnlock()
+
+	// Stop the task
+	cancel()
+	err = <-updateCacheTaskErr
+	require.ErrorIs(t, err, context.Canceled)
+}
+
 func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 	log, hook := test.NewNullLogger()
@@ -213,7 +293,16 @@ func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 	ds := fakedatastore.New(t)
 	metrics := fakemetrics.New()
 
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	require.NoError(t, err)
 	require.NotNil(t, ef)
 
@@ -224,7 +313,7 @@ func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 	go func() {
 		updateCacheTaskErr <- ef.RunUpdateCacheTask(ctx)
 	}()
-	clk.WaitForAfter(time.Second, "waiting for initial task pause")
+	clk.WaitForTickerMulti(time.Second, 2, "waiting to create tickers")
 	entries, err := ef.FetchAuthorizedEntries(ctx, agentID)
 	assert.NoError(t, err)
 	require.Zero(t, entries)
@@ -236,7 +325,7 @@ func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 	})
 	assert.NoError(t, err)
 
-	_, err = ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+	entry, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
 		SpiffeId: "spiffe://example.org/workload",
 		ParentId: agentID.String(),
 		Selectors: []*common.Selector{
@@ -250,10 +339,11 @@ func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 
 	// Bump clock and rerun UpdateCacheTask
 	clk.Add(defaultCacheReloadInterval)
-	clk.WaitForAfter(time.Second, "waiting for task to pause after creating entries")
-	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
-	assert.NoError(t, err)
-	require.Equal(t, 1, len(entries))
+	require.EventuallyWithT(t, func(c *assert.CollectT) {
+		entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
+		assert.NoError(c, err)
+	}, time.Second, 50*time.Millisecond)
+	compareEntries(t, entries, entry)
 
 	// Make sure nothing was pruned yet
 	for _, entry := range hook.AllEntries() {
@@ -262,9 +352,10 @@ func TestRunUpdateCacheTaskPrunesExpiredAgents(t *testing.T) {
 
 	// Bump clock so entry expires and is pruned
 	clk.Add(defaultCacheReloadInterval)
-	clk.WaitForAfter(time.Second, "waiting for task to pause after expiring agent")
-	assert.Equal(t, 1, hook.LastEntry().Data["count"])
-	assert.Equal(t, "Pruned expired agents from entry cache", hook.LastEntry().Message)
+	require.EventuallyWithT(t, func(c *assert.CollectT) {
+		assert.Equal(c, 1, hook.LastEntry().Data["count"])
+		assert.Equal(c, "Pruned expired agents from entry cache", hook.LastEntry().Message)
+	}, time.Second, 50*time.Millisecond)
 
 	// Stop the task
 	cancel()
@@ -279,7 +370,16 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 	ds := fakedatastore.New(t)
 	metrics := fakemetrics.New()
 
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	require.NoError(t, err)
 	require.NotNil(t, ef)
 
@@ -291,7 +391,7 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 	require.Zero(t, entries)
 
 	// Create Initial Registration Entry
-	entry, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+	entry1, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
 		SpiffeId: "spiffe://example.org/workload",
 		ParentId: agentID.String(),
 		Selectors: []*common.Selector{
@@ -309,10 +409,10 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 
 	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
 	require.NoError(t, err)
-	require.Equal(t, 1, len(entries))
+	compareEntries(t, entries, entry1)
 
 	// Delete initial registration entry
-	_, err = ds.DeleteRegistrationEntry(ctx, entry.EntryId)
+	_, err = ds.DeleteRegistrationEntry(ctx, entry1.EntryId)
 	require.NoError(t, err)
 
 	// Delete the event for now and then add it back later to simulate out of order events
@@ -320,7 +420,7 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 	require.NoError(t, err)
 
 	// Create Second entry
-	_, err = ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+	entry2, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
 		SpiffeId: "spiffe://example.org/workload2",
 		ParentId: agentID.String(),
 		Selectors: []*common.Selector{
@@ -338,12 +438,12 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 
 	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
 	require.NoError(t, err)
-	require.Equal(t, 2, len(entries))
+	compareEntries(t, entries, entry1, entry2)
 
 	// Add back in deleted event
 	err = ds.CreateRegistrationEntryEventForTesting(ctx, &datastore.RegistrationEntryEvent{
 		EventID: 2,
-		EntryID: entry.EntryId,
+		EntryID: entry1.EntryId,
 	})
 	require.NoError(t, err)
 
@@ -353,7 +453,7 @@ func TestUpdateRegistrationEntriesCacheSkippedEvents(t *testing.T) {
 
 	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
 	require.NoError(t, err)
-	require.Equal(t, 1, len(entries))
+	compareEntries(t, entries, entry2)
 }
 
 func TestUpdateRegistrationEntriesCacheSkippedStartupEvents(t *testing.T) {
@@ -403,7 +503,16 @@ func TestUpdateRegistrationEntriesCacheSkippedStartupEvents(t *testing.T) {
 	require.NoError(t, err)
 
 	// Create entry fetcher
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	require.NoError(t, err)
 	require.NotNil(t, ef)
 
@@ -476,7 +585,16 @@ func TestUpdateAttestedNodesCacheSkippedEvents(t *testing.T) {
 	ds := fakedatastore.New(t)
 	metrics := fakemetrics.New()
 
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
 	require.NoError(t, err)
 	require.NotNil(t, ef)
 
@@ -581,9 +699,7 @@ func TestUpdateAttestedNodesCacheSkippedEvents(t *testing.T) {
 
 	entries, err = ef.FetchAuthorizedEntries(ctx, agent2)
 	require.NoError(t, err)
-	require.Equal(t, 1, len(entries))
-	require.Equal(t, entry.EntryId, entries[0].GetId())
-	require.Equal(t, entry.SpiffeId, idutil.RequireIDProtoString(entries[0].GetSpiffeId()))
+	compareEntries(t, entries, entry)
 }
 
 func TestUpdateAttestedNodesCacheSkippedStartupEvents(t *testing.T) {
@@ -655,7 +771,15 @@ func TestUpdateAttestedNodesCacheSkippedStartupEvents(t *testing.T) {
 	require.NoError(t, err)
 
 	// Create entry fetcher
-	ef, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, log, metrics, clk, ds, defaultCacheReloadInterval, defaultPruneEventsOlderThan, defaultEventTimeout)
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                  log,
+		metrics:              metrics,
+		clk:                  clk,
+		ds:                   ds,
+		cacheReloadInterval:  defaultCacheReloadInterval,
+		pruneEventsOlderThan: defaultPruneEventsOlderThan,
+		eventTimeout:         defaultEventTimeout,
+	})
 	require.NoError(t, err)
 	require.NotNil(t, ef)
 
@@ -712,9 +836,203 @@ func TestUpdateAttestedNodesCacheSkippedStartupEvents(t *testing.T) {
 
 	entries, err = ef.FetchAuthorizedEntries(ctx, agent1)
 	require.NoError(t, err)
-	require.Equal(t, 1, len(entries))
-	require.Equal(t, entry.EntryId, entries[0].GetId())
-	require.Equal(t, entry.SpiffeId, idutil.RequireIDProtoString(entries[0].GetSpiffeId()))
+	compareEntries(t, entries, entry)
+}
+
+func TestFullCacheReloadRecoversFromSkippedRegistrationEntryEvents(t *testing.T) {
+	ctx := context.Background()
+	log, _ := test.NewNullLogger()
+	clk := clock.NewMock(t)
+	ds := fakedatastore.New(t)
+	metrics := fakemetrics.New()
+
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
+	require.NoError(t, err)
+	require.NotNil(t, ef)
+
+	agentID := spiffeid.RequireFromString("spiffe://example.org/myagent")
+
+	// Ensure no entries are in there to start
+	entries, err := ef.FetchAuthorizedEntries(ctx, agentID)
+	require.NoError(t, err)
+	require.Zero(t, entries)
+
+	// Create Initial Registration Entry
+	entry1, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+		SpiffeId: "spiffe://example.org/workload",
+		ParentId: agentID.String(),
+		Selectors: []*common.Selector{
+			{
+				Type:  "workload",
+				Value: "one",
+			},
+		},
+	})
+	require.NoError(t, err)
+
+	// Ensure it gets added to cache
+	err = ef.updateCache(ctx)
+	require.NoError(t, err)
+
+	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
+	require.NoError(t, err)
+	compareEntries(t, entries, entry1)
+
+	// Create Second entry
+	entry2, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+		SpiffeId: "spiffe://example.org/workload2",
+		ParentId: agentID.String(),
+		Selectors: []*common.Selector{
+			{
+				Type:  "workload",
+				Value: "two",
+			},
+		},
+	})
+	require.NoError(t, err)
+
+	// Delete the event
+	err = ds.DeleteRegistrationEntryEventForTesting(ctx, 2)
+	require.NoError(t, err)
+
+	// Check second entry is not added to cache
+	err = ef.updateCache(ctx)
+	require.NoError(t, err)
+
+	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
+	require.NoError(t, err)
+	compareEntries(t, entries, entry1)
+
+	// Rebuild the cache
+	err = ef.buildCache(ctx)
+	require.NoError(t, err)
+
+	// Should be 2 entries now
+	entries, err = ef.FetchAuthorizedEntries(ctx, agentID)
+	require.NoError(t, err)
+	compareEntries(t, entries, entry1, entry2)
+}
+
+func TestFullCacheReloadRecoversFromSkippedAttestedNodeEvents(t *testing.T) {
+	ctx := context.Background()
+	log, _ := test.NewNullLogger()
+	clk := clock.NewMock(t)
+	ds := fakedatastore.New(t)
+	metrics := fakemetrics.New()
+
+	ef, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+		log:                     log,
+		metrics:                 metrics,
+		clk:                     clk,
+		ds:                      ds,
+		cacheReloadInterval:     defaultCacheReloadInterval,
+		fullCacheReloadInterval: defaultFullCacheReloadInterval,
+		pruneEventsOlderThan:    defaultPruneEventsOlderThan,
+		eventTimeout:            defaultEventTimeout,
+	})
+	require.NoError(t, err)
+	require.NotNil(t, ef)
+
+	agent1 := spiffeid.RequireFromString("spiffe://example.org/myagent1")
+	agent2 := spiffeid.RequireFromString("spiffe://example.org/myagent2")
+
+	// Ensure no entries are in there to start
+	entries, err := ef.FetchAuthorizedEntries(ctx, agent2)
+	require.NoError(t, err)
+	require.Zero(t, entries)
+
+	// Create node alias for agent 2
+	alias, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+		SpiffeId: "spiffe://example.org/alias",
+		ParentId: "spiffe://example.org/spire/server",
+		Selectors: []*common.Selector{
+			{
+				Type:  "test",
+				Value: "alias",
+			},
+		},
+	})
+	assert.NoError(t, err)
+
+	// Create a registration entry parented to the alias
+	entry, err := ds.CreateRegistrationEntry(ctx, &common.RegistrationEntry{
+		SpiffeId: "spiffe://example.org/viaalias",
+		ParentId: alias.SpiffeId,
+		Selectors: []*common.Selector{
+			{
+				Type:  "workload",
+				Value: "two",
+			},
+		},
+	})
+	assert.NoError(t, err)
+
+	// Create both Attested Nodes
+	_, err = ds.CreateAttestedNode(ctx, &common.AttestedNode{
+		SpiffeId:     agent1.String(),
+		CertNotAfter: time.Now().Add(5 * time.Hour).Unix(),
+	})
+	require.NoError(t, err)
+
+	_, err = ds.CreateAttestedNode(ctx, &common.AttestedNode{
+		SpiffeId:     agent2.String(),
+		CertNotAfter: time.Now().Add(5 * time.Hour).Unix(),
+	})
+	require.NoError(t, err)
+
+	// Create selectors for agent 2
+	err = ds.SetNodeSelectors(ctx, agent2.String(), []*common.Selector{
+		{
+			Type:  "test",
+			Value: "alias",
+		},
+		{
+			Type:  "test",
+			Value: "cluster2",
+		},
+	})
+	assert.NoError(t, err)
+
+	// Create selectors for agent 1
+	err = ds.SetNodeSelectors(ctx, agent1.String(), []*common.Selector{
+		{
+			Type:  "test",
+			Value: "cluster1",
+		},
+	})
+	assert.NoError(t, err)
+
+	// Delete the events for agent 2 for now and then add it back later to simulate out of order events
+	err = ds.DeleteAttestedNodeEventForTesting(ctx, 2)
+	require.NoError(t, err)
+	err = ds.DeleteAttestedNodeEventForTesting(ctx, 3)
+	require.NoError(t, err)
+
+	// Should not be in cache yet
+	err = ef.updateCache(ctx)
+	require.NoError(t, err)
+
+	entries, err = ef.FetchAuthorizedEntries(ctx, agent2)
+	require.NoError(t, err)
+	require.Len(t, entries, 0)
+
+	// Do full reload
+	err = ef.buildCache(ctx)
+	require.NoError(t, err)
+
+	// Make sure it gets processed and the initial entry is deleted
+	entries, err = ef.FetchAuthorizedEntries(ctx, agent2)
+	require.NoError(t, err)
+	compareEntries(t, entries, entry)
 }
 
 // AgentsByIDCacheCount
@@ -788,3 +1106,20 @@ func entriesSkippedEventMetric(val float32) fakemetrics.MetricItem {
 		Labels: nil,
 	}
 }
+
+func compareEntries(t *testing.T, authorizedEntries []api.ReadOnlyEntry, entries ...*common.RegistrationEntry) {
+	t.Helper()
+
+	require.Equal(t, len(authorizedEntries), len(entries))
+	entryIDs := make([]string, 0, len(authorizedEntries))
+	spiffeIDs := make([]string, 0, len(authorizedEntries))
+	for _, entry := range authorizedEntries {
+		entryIDs = append(entryIDs, entry.GetId())
+		spiffeIDs = append(spiffeIDs, idutil.RequireIDProtoString(entry.GetSpiffeId()))
+	}
+
+	for _, entry := range entries {
+		require.Contains(t, entryIDs, entry.EntryId)
+		require.Contains(t, spiffeIDs, entry.SpiffeId)
+	}
+}
