@@ -51,6 +51,10 @@ const (
 	// entry cache.
 	defaultCacheReloadInterval = 5 * time.Second
 
+	// This is the default amount of time between full refreshes of the in-memory
+	// entry cache.
+	defaultFullCacheReloadInterval = 24 * time.Hour
+
 	// This is the default amount of time events live before they are pruned
 	defaultPruneEventsOlderThan = 12 * time.Hour
 
@@ -69,6 +73,9 @@ type Server interface {
 	// canceled, the function returns nil. Otherwise, the error from the failed
 	// server is returned.
 	ListenAndServe(ctx context.Context) error
+
+	// WaitForListening blocks until the server starts listening.
+	WaitForListening()
 }
 
 type Endpoints struct {
@@ -90,6 +97,11 @@ type Endpoints struct {
 	AuthPolicyEngine             *authpolicy.Engine
 	AdminIDs                     []spiffeid.ID
 	TLSPolicy                    tlspolicy.Policy
+
+	hooks struct {
+		// test hook used to indicate that is listening
+		listening chan struct{}
+	}
 }
 
 type APIServers struct {
@@ -126,6 +138,15 @@ func New(ctx context.Context, c Config) (*Endpoints, error) {
 	if c.CacheReloadInterval == 0 {
 		c.CacheReloadInterval = defaultCacheReloadInterval
 	}
+
+	if c.FullCacheReloadInterval == 0 {
+		c.FullCacheReloadInterval = defaultFullCacheReloadInterval
+	}
+
+	if c.FullCacheReloadInterval <= c.CacheReloadInterval {
+		return nil, errors.New("full cache reload interval must be greater than cache reload interval")
+	}
+
 	if c.PruneEventsOlderThan == 0 {
 		c.PruneEventsOlderThan = defaultPruneEventsOlderThan
 	}
@@ -139,7 +160,16 @@ func New(ctx context.Context, c Config) (*Endpoints, error) {
 	var ef api.AuthorizedEntryFetcher
 	var cacheRebuildTask, pruneEventsTask func(context.Context) error
 	if c.EventsBasedCache {
-		efEventsBasedCache, err := NewAuthorizedEntryFetcherWithEventsBasedCache(ctx, c.Log, c.Metrics, c.Clock, ds, c.CacheReloadInterval, c.PruneEventsOlderThan, c.EventTimeout)
+		efEventsBasedCache, err := NewAuthorizedEntryFetcherEvents(ctx, AuthorizedEntryFetcherEventsConfig{
+			log:                     c.Log,
+			metrics:                 c.Metrics,
+			clk:                     c.Clock,
+			ds:                      ds,
+			cacheReloadInterval:     c.CacheReloadInterval,
+			fullCacheReloadInterval: c.FullCacheReloadInterval,
+			pruneEventsOlderThan:    c.PruneEventsOlderThan,
+			eventTimeout:            c.EventTimeout,
+		})
 		if err != nil {
 			return nil, err
 		}
@@ -183,6 +213,11 @@ func New(ctx context.Context, c Config) (*Endpoints, error) {
 		AuthPolicyEngine:             c.AuthPolicyEngine,
 		AdminIDs:                     c.AdminIDs,
 		TLSPolicy:                    c.TLSPolicy,
+		hooks: struct {
+			listening chan struct{}
+		}{
+			listening: make(chan struct{}),
+		},
 	}, nil
 }
 
@@ -330,6 +365,7 @@ func (e *Endpoints) runLocalAccess(ctx context.Context, server *grpc.Server) err
 
 	// Skip use of tomb here so we don't pollute a clean shutdown with errors
 	log.Info("Starting Server APIs")
+	e.triggerListeningHook()
 	errChan := make(chan error)
 	go func() { errChan <- server.Serve(l) }()
 
@@ -399,3 +435,18 @@ func (e *Endpoints) makeInterceptors() (grpc.UnaryServerInterceptor, grpc.Stream
 
 	return middleware.Interceptors(Middleware(log, e.Metrics, e.DataStore, clock.New(), e.RateLimit, e.AuthPolicyEngine, e.AuditLogEnabled, e.AdminIDs))
 }
+
+func (e *Endpoints) triggerListeningHook() {
+	if e.hooks.listening != nil {
+		e.hooks.listening <- struct{}{}
+	}
+}
+
+func (e *Endpoints) WaitForListening() {
+	if e.hooks.listening == nil {
+		e.Log.Warn("Listening hook not initialized, cannot wait for listening")
+		return
+	}
+
+	<-e.hooks.listening
+}
