@@ -32,6 +32,16 @@ type UpstreamAuthority interface {
 	// the upstream authority does not support streaming updates, the stream
 	// will return io.EOF when called.
 	PublishJWTKey(ctx context.Context, jwtKey *common.PublicKey) (jwtAuthorities []*common.PublicKey, stream UpstreamJWTAuthorityStream, err error)
+
+	// SubscribeToLocalBundle can be used to sync the local trust bundle with
+	// the upstream trust bundle.
+	// Support for this method is optional but strongly recommended.
+	// The function returns the latest set of upstream authorities and a
+	// stream for streaming upstream authority updates. The returned stream
+	// MUST be closed when the caller is no longer interested in updates. If
+	// the upstream authority does not support streaming updates, the stream
+	// will return io.EOF when called.
+	SubscribeToLocalBundle(ctx context.Context) (x509CAs []*x509certificate.X509Authority, jwtAuthorities []*common.PublicKey, stream LocalBundleUpdateStream, err error)
 }
 
 type UpstreamX509AuthorityStream interface {
@@ -48,9 +58,9 @@ type UpstreamX509AuthorityStream interface {
 }
 
 type UpstreamJWTAuthorityStream interface {
-	// RecvUpstreamJWTAuthorities returns the latest set of upstream X.509
+	// RecvUpstreamJWTAuthorities returns the latest set of upstream JWT
 	// authorities. The call blocks until the update is received, the Close()
-	// method is called, or the context originally passed into MintX509CA is
+	// method is called, or the context originally passed into PublishJWTKey is
 	// canceled. If the function returns an error, no more updates will be
 	// available over the stream.
 	RecvUpstreamJWTAuthorities() ([]*common.PublicKey, error)
@@ -59,3 +69,16 @@ type UpstreamJWTAuthorityStream interface {
 	// when they are done with the stream.
 	Close()
 }
+
+type LocalBundleUpdateStream interface {
+	// RecvLocalBundleUpdate returns the latest local trust domain bundle
+	// The call blocks until the update is received, the Close()
+	// method is called, or the context originally passed into GetTrustBundle is
+	// canceled. If the function returns an error, no more updates will be
+	// available over the stream.
+	RecvLocalBundleUpdate() ([]*x509certificate.X509Authority, []*common.PublicKey, error)
+
+	// Close() closes the stream. It MUST be called by callers of GetTrustBundle
+	// when they are done with the stream.
+	Close()
+}
