@@ -36,6 +36,7 @@ import (
 	"github.com/spiffe/spire/pkg/server/endpoints"
 	"github.com/spiffe/spire/pkg/server/hostservice/agentstore"
 	"github.com/spiffe/spire/pkg/server/hostservice/identityprovider"
+	"github.com/spiffe/spire/pkg/server/node"
 	"github.com/spiffe/spire/pkg/server/plugin/bundlepublisher"
 	"github.com/spiffe/spire/pkg/server/registration"
 	"github.com/spiffe/spire/pkg/server/svid"
@@ -212,14 +213,28 @@ func (s *Server) run(ctx context.Context) (err error) {
 		registrationManager.Run,
 		bundlePublishingManager.Run,
 		catalog.ReconfigureTask(s.config.Log.WithField(telemetry.SubsystemName, "reconfigurer"), cat),
-		healthChecker.ListenAndServe,
 	}
 
 	if s.config.LogReopener != nil {
 		tasks = append(tasks, s.config.LogReopener)
 	}
 
-	err = util.RunTasks(ctx, tasks...)
+	if s.config.PruneAttestedNodesExpiredFor != 0 {
+		nodeManager := s.newNodeManager(cat, metrics)
+		tasks = append(tasks, nodeManager.Run)
+	}
+
+	ctx, cancel := context.WithCancelCause(ctx)
+	defer cancel(nil)
+	taskRunner := util.NewTaskRunner(ctx, cancel)
+	taskRunner.StartTasks(tasks...)
+
+	// Wait for the server to start listening before proceeding with health
+	// checks.
+	endpointsServer.WaitForListening()
+
+	taskRunner.StartTasks(healthChecker.ListenAndServe)
+	err = taskRunner.Wait()
 	if errors.Is(err, context.Canceled) {
 		err = nil
 	}
@@ -299,16 +314,15 @@ func (s *Server) loadCatalog(ctx context.Context, metrics telemetry.Metrics, ide
 
 func (s *Server) newCredBuilder(cat catalog.Catalog) (*credtemplate.Builder, error) {
 	return credtemplate.NewBuilder(credtemplate.Config{
-		TrustDomain:                  s.config.TrustDomain,
-		X509CASubject:                s.config.CASubject,
-		X509CATTL:                    s.config.CATTL,
-		AgentSVIDTTL:                 s.config.AgentTTL,
-		X509SVIDTTL:                  s.config.X509SVIDTTL,
-		JWTSVIDTTL:                   s.config.JWTSVIDTTL,
-		JWTIssuer:                    s.config.JWTIssuer,
-		CredentialComposers:          cat.GetCredentialComposers(),
-		UseLegacyDownstreamX509CATTL: s.config.UseLegacyDownstreamX509CATTL,
-		TLSPolicy:                    s.config.TLSPolicy,
+		TrustDomain:         s.config.TrustDomain,
+		X509CASubject:       s.config.CASubject,
+		X509CATTL:           s.config.CATTL,
+		AgentSVIDTTL:        s.config.AgentTTL,
+		X509SVIDTTL:         s.config.X509SVIDTTL,
+		JWTSVIDTTL:          s.config.JWTSVIDTTL,
+		JWTIssuer:           s.config.JWTIssuer,
+		CredentialComposers: cat.GetCredentialComposers(),
+		TLSPolicy:           s.config.TLSPolicy,
 	})
 }
 
@@ -371,6 +385,19 @@ func (s *Server) newRegistrationManager(cat catalog.Catalog, metrics telemetry.M
 	return registrationManager
 }
 
+func (s *Server) newNodeManager(cat catalog.Catalog, metrics telemetry.Metrics) *node.Manager {
+	nodeManager := node.NewManager(node.ManagerConfig{
+		DataStore: cat.GetDataStore(),
+		Log:       s.config.Log.WithField(telemetry.SubsystemName, telemetry.NodeManager),
+		Metrics:   metrics,
+		PruneArgs: node.PruneArgs{
+			ExpiredFor:             s.config.PruneAttestedNodesExpiredFor,
+			IncludeNonReattestable: s.config.PruneNonReattestableNodes,
+		},
+	})
+	return nodeManager
+}
+
 func (s *Server) newSVIDRotator(ctx context.Context, serverCA ca.ServerCA, metrics telemetry.Metrics) (*svid.Rotator, error) {
 	svidRotator := svid.NewRotator(&svid.RotatorConfig{
 		ServerCA: serverCA,
@@ -386,28 +413,28 @@ func (s *Server) newSVIDRotator(ctx context.Context, serverCA ca.ServerCA, metri
 
 func (s *Server) newEndpointsServer(ctx context.Context, catalog catalog.Catalog, svidObserver svid.Observer, serverCA ca.ServerCA, metrics telemetry.Metrics, authorityManager manager.AuthorityManager, authPolicyEngine *authpolicy.Engine, bundleManager *bundle_client.Manager) (endpoints.Server, error) {
 	config := endpoints.Config{
-		TCPAddr:                      s.config.BindAddress,
-		LocalAddr:                    s.config.BindLocalAddress,
-		SVIDObserver:                 svidObserver,
-		TrustDomain:                  s.config.TrustDomain,
-		Catalog:                      catalog,
-		ServerCA:                     serverCA,
-		Log:                          s.config.Log.WithField(telemetry.SubsystemName, telemetry.Endpoints),
-		RootLog:                      s.config.Log,
-		Metrics:                      metrics,
-		AuthorityManager:             authorityManager,
-		RateLimit:                    s.config.RateLimit,
-		Uptime:                       uptime.Uptime,
-		Clock:                        clock.New(),
-		CacheReloadInterval:          s.config.CacheReloadInterval,
-		EventsBasedCache:             s.config.EventsBasedCache,
-		PruneEventsOlderThan:         s.config.PruneEventsOlderThan,
-		EventTimeout:                 s.config.EventTimeout,
-		AuditLogEnabled:              s.config.AuditLogEnabled,
-		AuthPolicyEngine:             authPolicyEngine,
-		BundleManager:                bundleManager,
-		AdminIDs:                     s.config.AdminIDs,
-		UseLegacyDownstreamX509CATTL: s.config.UseLegacyDownstreamX509CATTL,
+		TCPAddr:                 s.config.BindAddress,
+		LocalAddr:               s.config.BindLocalAddress,
+		SVIDObserver:            svidObserver,
+		TrustDomain:             s.config.TrustDomain,
+		Catalog:                 catalog,
+		ServerCA:                serverCA,
+		Log:                     s.config.Log.WithField(telemetry.SubsystemName, telemetry.Endpoints),
+		RootLog:                 s.config.Log,
+		Metrics:                 metrics,
+		AuthorityManager:        authorityManager,
+		RateLimit:               s.config.RateLimit,
+		Uptime:                  uptime.Uptime,
+		Clock:                   clock.New(),
+		CacheReloadInterval:     s.config.CacheReloadInterval,
+		FullCacheReloadInterval: s.config.FullCacheReloadInterval,
+		EventsBasedCache:        s.config.EventsBasedCache,
+		PruneEventsOlderThan:    s.config.PruneEventsOlderThan,
+		EventTimeout:            s.config.EventTimeout,
+		AuditLogEnabled:         s.config.AuditLogEnabled,
+		AuthPolicyEngine:        authPolicyEngine,
+		BundleManager:           bundleManager,
+		AdminIDs:                s.config.AdminIDs,
 	}
 	if s.config.Federation.BundleEndpoint != nil {
 		config.BundleEndpoint.Address = s.config.Federation.BundleEndpoint.Address
